/*
Filter form
*/

#search-form-filter .form-group {
    margin-bottom: 0px;
}

#search-filter-input-terms {
    display: inline-block;
}

/*
Search form
*/

#search-navbar-form .form-control:focus {
    border-color: #dce4ec;
    box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 8px rgba(44, 62, 80, 0.6);
}

#search-navbar-form .form-control {
    border-color: #2c3e50;
}

#search-navbar-form input {
    margin-right: 2px;
}

#search-navbar-form select {
    margin-right: 2px;
}

#search-navbar-form button {
    border: 1px solid #ecf0f1;
    margin-right: 4px;
}

#search-navbar-form a {
    border: 1px solid #ecf0f1;
    margin-right: 4px;
}
