{% load i18n %}

{% load databases_tags %}

{# appearance_app_template_nocache #}

{% databases_check_sqlite as check_sqlite %}
{% if check_sqlite %}
    <div class="alert alert-dismissible alert-warning">
        <button type="button" class="close" data-dismiss="alert">&times;</button>
        <p><strong>{% trans 'Warning' %}</strong> {{ check_sqlite }}</p>
    </div>
{% endif %}

