{% spaceless %}
<div class="text-center" style="border: height: 100%;">
    <div class="fa-4x">
        <span class="fa-layers fa-fw">
            <i class="far fa-file text-primary"></i>
            <i class="fas fa-times text-danger" data-fa-transform="shrink-8"></i>
        </span>
    </div>
</div>
{% endspaceless %}

