TEST_CASE_ACTION_NAME_SOURCE_CREATE = 'source-create'

TEST_CASE_INTERFACE_NAME_MODEL = 'model'
TEST_CASE_INTERFACE_NAME_REST_API = 'rest-api'
TEST_CASE_INTERFACE_NAME_VIEW = 'view'

TEST_SOURCE_ACTION_CONFIRM_FALSE_NAME = 'action_test_no_confirm'
TEST_SOURCE_ACTION_CONFIRM_TRUE_NAME = 'action_test_confirm'

TEST_SOURCE_BACKEND_PATH_BASE = 'mayan.apps.sources.tests.source_backends.SourceBackendTest'
TEST_SOURCE_BACKEND_PATH_DOCUMENT_BASIC = 'mayan.apps.sources.tests.source_backends.SourceBackendDocumentUploadTest'

TEST_SOURCE_LABEL = 'test source'
TEST_SOURCE_LABEL_EDITED = 'test source edited'

TEST_SOURCE_METADATA_KEY = 'test_source_metadata_key'
TEST_SOURCE_METADATA_VALUE = 'test_source_metadata_value'
